@@ -24,7 +24,9 @@
  */
 
 #include <gtk/gtk.h>
+#include "idorange.h"
 #include "idoscalemenuitem.h"
+#include "idotypebuiltins.h"
 
 static void     ido_scale_menu_item_set_property           (GObject               *object,
                                                             guint                  prop_id,
@@ -49,18 +51,25 @@ static void     ido_scale_menu_item_secondary_image_notify (GtkImage
 static void     ido_scale_menu_item_notify                 (IdoScaleMenuItem      *item,
                                                             GParamSpec            *pspec,
                                                             gpointer               user_data);
+static void     update_packing                             (IdoScaleMenuItem      *self,
+                                                            IdoScaleMenuItemStyle  style,
+                                                            IdoScaleMenuItemStyle  old_style);
 
 struct _IdoScaleMenuItemPrivate {
-  GtkWidget     *scale;
-  GtkAdjustment *adjustment;
-  GtkWidget     *primary_image;
-  GtkWidget     *secondary_image;
-  GtkWidget     *hbox;
-  GtkAllocation  child_allocation;
-  gdouble        left_padding;
-  gdouble        right_padding;
-  gboolean       reverse_scroll;
-  gboolean       grabbed;
+  GtkWidget            *scale;
+  GtkAdjustment        *adjustment;
+  GtkWidget            *primary_image;
+  GtkWidget            *secondary_image;
+  GtkWidget            *primary_label;
+  GtkWidget            *secondary_label;
+  GtkWidget            *hbox;
+  GtkAllocation         child_allocation;
+  gdouble               left_padding;
+  gdouble               right_padding;
+  gboolean              reverse_scroll;
+  gboolean              grabbed;
+  IdoScaleMenuItemStyle style;
+  IdoRangeStyle         range_style;
 };
 
 enum {
@@ -72,7 +81,9 @@ enum {
 enum {
   PROP_0,
   PROP_ADJUSTMENT,
-  PROP_REVERSE_SCROLL_EVENTS
+  PROP_REVERSE_SCROLL_EVENTS,
+  PROP_STYLE,
+  PROP_RANGE_STYLE
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -123,23 +134,43 @@ ido_scale_menu_item_size_allocate (GtkWidget     *widget,
                                    GtkAllocation *allocation)
 {
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (widget);
-  GtkRequisition primary_image_req;
-  GtkRequisition secondary_image_req;
+  GtkRequisition primary_req;
+  GtkRequisition secondary_req;
   gint horizontal_padding;
   gint primary_padding, secondary_padding;
 
   GTK_WIDGET_CLASS (ido_scale_menu_item_parent_class)->size_allocate (widget, allocation);
 
-  gtk_widget_get_child_requisition (priv->primary_image, &primary_image_req);
-  gtk_widget_get_child_requisition (priv->secondary_image, &secondary_image_req);
+  switch (priv->style)
+    {
+    case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
+      gtk_widget_get_child_requisition (priv->primary_image, &primary_req);
+      gtk_widget_get_child_requisition (priv->secondary_image, &secondary_req);
+
+      primary_padding = gtk_widget_get_visible (priv->primary_image) ? primary_req.width : 0;
+      secondary_padding = gtk_widget_get_visible (priv->secondary_image) ? secondary_req.width : 0;
+      break;
+
+    case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
+      gtk_widget_get_child_requisition (priv->primary_label, &primary_req);
+      gtk_widget_get_child_requisition (priv->secondary_label, &secondary_req);
+
+      primary_padding = gtk_widget_get_visible (priv->primary_label) ? primary_req.width : 0;
+      secondary_padding = gtk_widget_get_visible (priv->secondary_label) ? secondary_req.width : 0;
+      break;
+
+    default:
+      primary_req.width = primary_req.height = 0;
+      secondary_req.width = secondary_req.height = 0;
+      primary_padding = 0;
+      secondary_padding = 0;
+      break;
+    }
 
   gtk_widget_style_get (widget,
                         "horizontal-padding", &horizontal_padding,
                         NULL);
 
-  primary_padding = gtk_widget_get_visible (priv->primary_image) ? primary_image_req.width : 0;
-  secondary_padding = gtk_widget_get_visible (priv->secondary_image) ? secondary_image_req.width : 0;
-
   priv->left_padding = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR ? primary_padding : secondary_padding;
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
@@ -166,6 +197,51 @@ ido_scale_menu_item_size_allocate (GtkWidget     *widget,
   gtk_widget_set_size_request (priv->scale, priv->child_allocation.width, -1);
 }
 
+static void
+ido_scale_menu_item_constructed (GObject *object)
+{
+  IdoScaleMenuItem *self = IDO_SCALE_MENU_ITEM (object);
+  IdoScaleMenuItemPrivate *priv = GET_PRIVATE (self);
+  GtkObject *adj = gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 0.0);
+  IdoRangeStyle range_style;
+  GtkWidget *hbox;
+
+  priv->adjustment  = NULL;
+
+  g_object_get (self,
+                "range-style", &range_style,
+                NULL);
+
+  priv->scale = ido_range_new (adj, range_style);
+  g_object_ref (priv->scale);
+  gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
+
+  hbox = gtk_hbox_new (FALSE, 0);
+
+  priv->primary_image = gtk_image_new ();
+  g_signal_connect (priv->primary_image, "notify",
+                    G_CALLBACK (ido_scale_menu_item_primary_image_notify),
+                    self);
+
+  priv->secondary_image = gtk_image_new ();
+  g_signal_connect (priv->secondary_image, "notify",
+                    G_CALLBACK (ido_scale_menu_item_secondary_image_notify),
+                    self);
+
+  priv->primary_label = gtk_label_new ("");
+  priv->secondary_label = gtk_label_new ("");
+
+  priv->hbox = hbox;
+
+  update_packing (self, priv->style, priv->style);
+
+  g_signal_connect (self, "notify",
+                    G_CALLBACK (ido_scale_menu_item_notify),
+                    NULL);
+
+  gtk_container_add (GTK_CONTAINER (self), hbox);
+}
+
 static void
 ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
 {
@@ -173,16 +249,35 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
   GtkObjectClass    *object_class = GTK_OBJECT_CLASS (item_class);
   GtkWidgetClass    *widget_class = GTK_WIDGET_CLASS (item_class);
 
-  widget_class->button_press_event = ido_scale_menu_item_button_press_event;
+  widget_class->button_press_event   = ido_scale_menu_item_button_press_event;
   widget_class->button_release_event = ido_scale_menu_item_button_release_event;
-  widget_class->motion_notify_event = ido_scale_menu_item_motion_notify_event;
-  widget_class->scroll_event = ido_scale_menu_item_scroll_event;
-  widget_class->state_changed = ido_scale_menu_item_state_changed;
-  widget_class->size_allocate = ido_scale_menu_item_size_allocate;
+  widget_class->motion_notify_event  = ido_scale_menu_item_motion_notify_event;
+  widget_class->scroll_event         = ido_scale_menu_item_scroll_event;
+  widget_class->state_changed        = ido_scale_menu_item_state_changed;
+  widget_class->size_allocate        = ido_scale_menu_item_size_allocate;
 
+  gobject_class->constructed  = ido_scale_menu_item_constructed;
   gobject_class->set_property = ido_scale_menu_item_set_property;
   gobject_class->get_property = ido_scale_menu_item_get_property;
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_STYLE,
+                                   g_param_spec_enum ("accessory-style",
+                                                      "Style of primary/secondary widgets",
+                                                      "The style of the primary/secondary widgets",
+                                                      IDO_TYPE_SCALE_MENU_ITEM_STYLE,
+                                                      IDO_SCALE_MENU_ITEM_STYLE_NONE,
+                                                      G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_RANGE_STYLE,
+                                   g_param_spec_enum ("range-style",
+                                                      "Range style",
+                                                      "Style of the range",
+                                                      IDO_TYPE_RANGE_STYLE,
+                                                      IDO_RANGE_STYLE_DEFAULT,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
   g_object_class_install_property (gobject_class,
                                    PROP_ADJUSTMENT,
                                    g_param_spec_object ("adjustment",
@@ -219,41 +314,66 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
 }
 
 static void
-ido_scale_menu_item_init (IdoScaleMenuItem *self)
+update_packing (IdoScaleMenuItem *self, IdoScaleMenuItemStyle style, IdoScaleMenuItemStyle old_style)
 {
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (self);
-  GtkWidget *hbox;
+  GtkContainer *container = GTK_CONTAINER (priv->hbox);
 
-  priv->adjustment = NULL;
+  if (style != old_style)
+    {
+      switch (old_style)
+        {
+        case IDO_SCALE_MENU_ITEM_STYLE_NONE:
+          gtk_container_remove (container, priv->scale);
+          break;
 
-  priv->scale = gtk_hscale_new_with_range (0.0, 100.0, 1.0);
-  gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
+        case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
+          gtk_container_remove (container, priv->primary_image);
+          gtk_container_remove (container, priv->secondary_image);
+          gtk_container_remove (container, priv->scale);
+          break;
 
-  hbox = gtk_hbox_new (FALSE, 0);
+        case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
+          gtk_container_remove (container, priv->primary_label);
+          gtk_container_remove (container, priv->secondary_label);
+          gtk_container_remove (container, priv->scale);
+          break;
 
-  priv->primary_image = gtk_image_new ();
-  g_signal_connect (priv->primary_image, "notify",
-                    G_CALLBACK (ido_scale_menu_item_primary_image_notify),
-                    self);
+        default:
+          gtk_container_remove (container, priv->scale);
+          break;
+        }
+    }
 
-  priv->secondary_image = gtk_image_new ();
-  g_signal_connect (priv->secondary_image, "notify",
-                    G_CALLBACK (ido_scale_menu_item_secondary_image_notify),
-                    self);
+  switch (style)
+    {
+    case IDO_SCALE_MENU_ITEM_STYLE_NONE:
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+      break;
 
-  gtk_box_pack_start (GTK_BOX (hbox), priv->primary_image, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), priv->scale, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), priv->secondary_image, FALSE, FALSE, 0);
+    case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->primary_image, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->secondary_image, FALSE, FALSE, 0);
+      break;
 
-  priv->hbox = hbox;
+    case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->primary_label, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->secondary_label, FALSE, FALSE, 0);
+      break;
+
+    default:
+      gtk_box_pack_start (GTK_BOX (priv->hbox), priv->scale, FALSE, FALSE, 0);
+      break;
+    }
 
   gtk_widget_show_all (priv->hbox);
+}
 
-  g_signal_connect (self, "notify",
-                    G_CALLBACK (ido_scale_menu_item_notify),
-                    NULL);
-
-  gtk_container_add (GTK_CONTAINER (self), hbox);
+static void
+ido_scale_menu_item_init (IdoScaleMenuItem *self)
+{
 }
 
 static void
@@ -275,6 +395,14 @@ ido_scale_menu_item_set_property (GObject         *object,
       priv->reverse_scroll = g_value_get_boolean (value);
       break;
 
+    case PROP_STYLE:
+      ido_scale_menu_item_set_style (menu_item, g_value_get_enum (value));
+      break;
+
+    case PROP_RANGE_STYLE:
+      priv->range_style = g_value_get_enum (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -289,11 +417,12 @@ ido_scale_menu_item_get_property (GObject         *object,
 {
   IdoScaleMenuItem *menu_item = IDO_SCALE_MENU_ITEM (object);
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (menu_item);
-  GtkAdjustment *adjustment = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  GtkAdjustment *adjustment;
 
   switch (prop_id)
     {
     case PROP_ADJUSTMENT:
+      adjustment = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
       g_value_set_object (value, adjustment);
       break;
 
@@ -301,6 +430,10 @@ ido_scale_menu_item_get_property (GObject         *object,
       g_value_set_boolean (value, priv->reverse_scroll);
       break;
 
+    case PROP_RANGE_STYLE:
+      g_value_set_enum (value, priv->range_style);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -335,12 +468,12 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
   translate_event_coordinates (menuitem, event->x_root, &x);
   event->x_root = x;
 
-  GTK_OBJECT_FLAGS (scale) |= GTK_HAS_GRAB;
+  ubuntu_gtk_widget_set_has_grab (scale, TRUE);
 
   gtk_widget_event (scale,
                     ((GdkEvent *)(void*)(event)));
 
-  GTK_OBJECT_FLAGS (scale) &= ~(GTK_HAS_GRAB);
+  ubuntu_gtk_widget_set_has_grab (scale, FALSE);
 
   if (!priv->grabbed)
     {
@@ -504,6 +637,7 @@ ido_scale_menu_item_secondary_image_notify (GtkImage         *image,
 /**
  * ido_scale_menu_item_new:
  * @label: the text of the new menu item.
+ * @size: The size style of the range.
  * @adjustment: A #GtkAdjustment describing the slider value.
  * @returns: a new #IdoScaleMenuItem.
  *
@@ -511,16 +645,19 @@ ido_scale_menu_item_secondary_image_notify (GtkImage         *image,
  **/
 GtkWidget*
 ido_scale_menu_item_new (const gchar   *label,
+                         IdoRangeStyle  range_style,
                          GtkAdjustment *adjustment)
 {
   return g_object_new (IDO_TYPE_SCALE_MENU_ITEM,
-                       "adjustment", adjustment,
+                       "adjustment",  adjustment,
+                       "range-style", range_style,
                        NULL);
 }
 
 /**
  * ido_scale_menu_item_new_with_label:
  * @label: the text of the menu item.
+ * @size: The size style of the range.
  * @min: The minimum value of the slider.
  * @max: The maximum value of the slider.
  * @step: The step increment of the slider.
@@ -529,20 +666,29 @@ ido_scale_menu_item_new (const gchar   *label,
  * Creates a new #IdoScaleMenuItem containing a label.
  **/
 GtkWidget*
-ido_scale_menu_item_new_with_range (const gchar *label,
-                                    gdouble      value,
-                                    gdouble      min,
-                                    gdouble      max,
-                                    gdouble      step)
+ido_scale_menu_item_new_with_range (const gchar  *label,
+                                    IdoRangeStyle range_style,
+                                    gdouble       value,
+                                    gdouble       min,
+                                    gdouble       max,
+                                    gdouble       step)
 {
   GtkObject *adjustment = gtk_adjustment_new (value, min, max, step, 10 * step, 0);
 
   return g_object_new (IDO_TYPE_SCALE_MENU_ITEM,
-                       "label", label,
-                       "adjustment", adjustment,
+                       "label",       label,
+                       "range-style", range_style,
+                       "adjustment",  adjustment,
 		       NULL);
 }
 
+/**
+ * ido_scale_menu_item_get_scale:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A pointer to the scale widget.
+ *
+ * Retrieves the scale widget.
+ **/
 GtkWidget*
 ido_scale_menu_item_get_scale (IdoScaleMenuItem *menuitem)
 {
@@ -555,6 +701,52 @@ ido_scale_menu_item_get_scale (IdoScaleMenuItem *menuitem)
   return priv->scale;
 }
 
+/**
+ * ido_scale_menu_item_get_style:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A #IdoScaleMenuItemStyle enum describing the style.
+ *
+ * Retrieves the type of widgets being used for the primary and
+ * secondary widget slots.  This could be images, labels, or nothing.
+ **/
+IdoScaleMenuItemStyle
+ido_scale_menu_item_get_style (IdoScaleMenuItem *menuitem)
+{
+  IdoScaleMenuItemPrivate *priv;
+
+  g_return_val_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem), IDO_SCALE_MENU_ITEM_STYLE_NONE);
+
+  priv = GET_PRIVATE (menuitem);
+
+  return priv->style;
+}
+
+void
+ido_scale_menu_item_set_style (IdoScaleMenuItem      *menuitem,
+                               IdoScaleMenuItemStyle  style)
+{
+  IdoScaleMenuItemPrivate *priv;
+  IdoScaleMenuItemStyle    old_style;
+
+  g_return_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem));
+
+  priv = GET_PRIVATE (menuitem);
+
+  old_style = priv->style;
+  priv->style = style;
+
+  update_packing (menuitem, style, old_style);
+}
+
+/**
+ * ido_scale_menu_item_get_primary_image:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A #GtkWidget pointer for the primary image.
+ *
+ * Retrieves a pointer to the image widget used in the primary slot.
+ * Whether this is visible depends upon the return value from
+ * ido_scale_menu_item_get_style().
+ **/
 GtkWidget *
 ido_scale_menu_item_get_primary_image (IdoScaleMenuItem *menuitem)
 {
@@ -567,6 +759,15 @@ ido_scale_menu_item_get_primary_image (IdoScaleMenuItem *menuitem)
   return priv->primary_image;
 }
 
+/**
+ * ido_scale_menu_item_get_secondary_image:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A #GtkWidget pointer for the secondary image.
+ *
+ * Retrieves a pointer to the image widget used in the secondary slot.
+ * Whether this is visible depends upon the return value from
+ * ido_scale_menu_item_get_style().
+ **/
 GtkWidget *
 ido_scale_menu_item_get_secondary_image (IdoScaleMenuItem *menuitem)
 {
@@ -579,5 +780,97 @@ ido_scale_menu_item_get_secondary_image (IdoScaleMenuItem *menuitem)
   return priv->secondary_image;
 }
 
+/**
+ * ido_scale_menu_item_get_primary_label:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A const gchar* string of the label text.
+ *
+ * Retrieves a string of the text for the primary label widget.
+ * Whether this is visible depends upon the return value from
+ * ido_scale_menu_item_get_style().
+ **/
+G_CONST_RETURN gchar*
+ido_scale_menu_item_get_primary_label (IdoScaleMenuItem *menuitem)
+{
+  IdoScaleMenuItemPrivate *priv;
+
+  g_return_val_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem), NULL);
+
+  priv = GET_PRIVATE (menuitem);
+
+  return gtk_label_get_text (GTK_LABEL (priv->primary_label));
+}
+
+/**
+ * ido_scale_menu_item_get_primary_label:
+ * @menuitem: The #IdoScaleMenuItem
+ * @returns: A const gchar* string of the label text.
+ *
+ * Retrieves a string of the text for the primary label widget.
+ * Whether this is visible depends upon the return value from
+ * ido_scale_menu_item_get_style().
+ **/
+G_CONST_RETURN gchar*
+ido_scale_menu_item_get_secondary_label (IdoScaleMenuItem *menuitem)
+{
+  IdoScaleMenuItemPrivate *priv;
+
+  g_return_val_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem), NULL);
+
+  priv = GET_PRIVATE (menuitem);
+
+  return gtk_label_get_text (GTK_LABEL (priv->secondary_label));
+}
+
+/**
+ * ido_scale_menu_item_set_primary_label:
+ * @menuitem: The #IdoScaleMenuItem
+ * @label: A string containing the label text
+ *
+ * Sets the text for the label widget in the primary slot.  This
+ * widget will only be visibile if the return value of
+ * ido_scale_menu_item_get_style() is set to %IDO_SCALE_MENU_ITEM_STYLE_LABEL.
+ **/
+void
+ido_scale_menu_item_set_primary_label (IdoScaleMenuItem *menuitem,
+                                       const gchar      *label)
+{
+  IdoScaleMenuItemPrivate *priv;
+
+  g_return_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem));
+
+  priv = GET_PRIVATE (menuitem);
+
+  if (priv->primary_label)
+    {
+      gtk_label_set_text (GTK_LABEL (priv->primary_label), label);
+    }
+}
+
+/**
+ * ido_scale_menu_item_set_primary_label:
+ * @menuitem: The #IdoScaleMenuItem
+ * @label: A string containing the label text
+ *
+ * Sets the text for the label widget in the primary slot.  This
+ * widget will only be visibile if the return value of
+ * ido_scale_menu_item_get_style() is set to %IDO_SCALE_MENU_ITEM_STYLE_LABEL.
+ **/
+void
+ido_scale_menu_item_set_secondary_label (IdoScaleMenuItem *menuitem,
+                                         const gchar      *label)
+{
+  IdoScaleMenuItemPrivate *priv;
+
+  g_return_if_fail (IDO_IS_SCALE_MENU_ITEM (menuitem));
+
+  priv = GET_PRIVATE (menuitem);
+
+  if (priv->secondary_label)
+    {
+      gtk_label_set_text (GTK_LABEL (priv->secondary_label), label);
+    }
+}
+
 
 #define __IDO_SCALE_MENU_ITEM_C__
